@@ -144,7 +144,8 @@ public:
   }
 
 #ifdef __SYCL_DEVICE_ONLY__
-  explicit atomic_ref_base(T &ref) : ptr(multi_ptr<T, AddressSpace>(&ref)) {}
+  explicit atomic_ref_base(T &ref)
+      : ptr(address_space_cast<AddressSpace, access::decorated::no>(&ref)) {}
 #else
   // FIXME: This reinterpret_cast is UB, but happens to work for now
   explicit atomic_ref_base(T &ref)
@@ -245,7 +246,7 @@ public:
 
 protected:
 #ifdef __SYCL_DEVICE_ONLY__
-  multi_ptr<T, AddressSpace> ptr;
+  multi_ptr<T, AddressSpace, access::decorated::no> ptr;
 #else
   std::atomic<T> *ptr;
 #endif
